@@ -1,7 +1,8 @@
 
 --- Module implementing the LuaRocks "search" command.
 -- Queries LuaRocks servers.
-module("luarocks.search", package.seeall)
+local search = {}
+package.loaded["luarocks.search"] = search
 
 local dir = require("luarocks.dir")
 local path = require("luarocks.path")
@@ -10,9 +11,10 @@ local deps = require("luarocks.deps")
 local cfg = require("luarocks.cfg")
 local util = require("luarocks.util")
 
-help_summary = "Query the LuaRocks servers."
-help_arguments = "[--source] [--binary] { <name> [<version>] | --all }"
-help = [[
+util.add_run_function(search)
+search.help_summary = "Query the LuaRocks servers."
+search.help_arguments = "[--source] [--binary] { <name> [<version>] | --all }"
+search.help = [[
 --source  Return only rockspecs and source rocks,
           to be used with the "build" command.
 --binary  Return only pure Lua and binary rocks (rocks that can be used
@@ -46,7 +48,8 @@ end
 
 --- Store a search result (a rock or rockspec) in the results table.
 -- @param results table: The results table, where keys are package names and
--- versions are tables matching version strings to an array of servers.
+-- values are tables matching version strings to arrays of
+-- tables with fields "arch" and "repo".
 -- @param name string: Package name.
 -- @param version string: Package version.
 -- @param arch string: Architecture of rock ("all", "src" or platform
@@ -90,7 +93,8 @@ end
 -- table, optionally checking if version and arch (if given) match
 -- a query.
 -- @param results table: The results table, where keys are package names and
--- versions are tables matching version strings to an array of servers.
+-- values are tables matching version strings to arrays of
+-- tables with fields "arch" and "repo".
 -- @param repo string: URL or pathname of the repository.
 -- @param name string: The name of the package being tested.
 -- @param version string: The version of the package being tested.
@@ -121,10 +125,11 @@ end
 -- matches regardless of architecture.
 -- @param results table or nil: If given, this table will store the
 -- results; if not given, a new table will be created.
--- @param table: The results table, where keys are package names and
--- versions are tables matching version strings to an array of servers.
+-- @return table: The results table, where keys are package names and
+-- values are tables matching version strings to arrays of
+-- tables with fields "arch" and "repo".
 -- If a table was given in the "results" parameter, that is the result value.
-function disk_search(repo, query, results)
+function search.disk_search(repo, query, results)
    assert(type(repo) == "string")
    assert(type(query) == "table")
    assert(type(results) == "table" or not results)
@@ -136,14 +141,14 @@ function disk_search(repo, query, results)
    end
    query_arch_as_table(query)
    
-   for _, name in pairs(fs.list_dir(repo)) do
+   for name in fs.dir(repo) do
       local pathname = dir.path(repo, name)
       local rname, rversion, rarch = path.parse_name(name)
 
       if rname and (pathname:match(".rockspec$") or pathname:match(".rock$")) then
          store_if_match(results, repo, rname, rversion, rarch, query)
       elseif fs.is_dir(pathname) then
-         for _, version in pairs(fs.list_dir(pathname)) do
+         for version in fs.dir(pathname) do
             if version:match("-%d+$") then
                store_if_match(results, repo, name, version, "installed", query)
             end
@@ -153,26 +158,29 @@ function disk_search(repo, query, results)
    return results
 end
 
---- Perform search on a rocks server.
+--- Perform search on a rocks server or tree.
 -- @param results table: The results table, where keys are package names and
--- versions are tables matching version strings to an array of servers.
--- @param repo string: The URL of the rocks server.
+-- values are tables matching version strings to arrays of
+-- tables with fields "arch" and "repo".
+-- @param repo string: The URL of a rocks server or
+-- the pathname of a rocks tree (as returned by path.rocks_dir()).
 -- @param query table: A table describing the query in dependency
 -- format (for example, {name = "filesystem", exact_name = false,
 -- constraints = {op = "~>", version = {1,0}}}, arch = "rockspec").
 -- If the arch field is omitted, the local architecture (cfg.arch)
 -- is used. The special value "any" is also recognized, returning all
 -- matches regardless of architecture.
--- @return true or, in case of errors, nil and an error message.
-function manifest_search(results, repo, query)
+-- @param lua_version string: Lua version in "5.x" format, defaults to installed version.
+-- @return true or, in case of errors, nil, an error message and an optional error code.
+function search.manifest_search(results, repo, query, lua_version)
    assert(type(results) == "table")
    assert(type(repo) == "string")
    assert(type(query) == "table")
    
    query_arch_as_table(query)
-   local manifest, err = manif.load_manifest(repo)
+   local manifest, err, errcode = manif.load_manifest(repo, lua_version)
    if not manifest then
-      return nil, "Failed loading manifest: "..err
+      return nil, err, errcode
    end
    for name, versions in pairs(manifest.repository) do
       for version, items in pairs(versions) do
@@ -186,33 +194,38 @@ end
 
 --- Search on all configured rocks servers.
 -- @param query table: A dependency query.
+-- @param lua_version string: Lua version in "5.x" format, defaults to installed version.
 -- @return table: A table where keys are package names
--- and values are tables matching version strings to an array of
--- rocks servers; if no results are found, an empty table is returned.
-function search_repos(query)
+-- and values are tables matching version strings to arrays of
+-- tables with fields "arch" and "repo".
+function search.search_repos(query, lua_version)
    assert(type(query) == "table")
 
    local results = {}
    for _, repo in ipairs(cfg.rocks_servers) do
-      if type(repo) == "string" then
-         repo = { repo }
-      end
-      for _, mirror in ipairs(repo) do
-         local protocol, pathname = dir.split_url(mirror)
-         if protocol == "file" then
-            mirror = pathname
+      if not cfg.disabled_servers[repo] then
+         if type(repo) == "string" then
+            repo = { repo }
          end
-         local ok, err = manifest_search(results, mirror, query)
-         if ok then
-            break
-         else
-            util.warning("Failed searching manifest: "..err)
+         for _, mirror in ipairs(repo) do
+            local protocol, pathname = dir.split_url(mirror)
+            if protocol == "file" then
+               mirror = pathname
+            end
+            local ok, err, errcode = search.manifest_search(results, mirror, query, lua_version)
+            if errcode == "network" then
+               cfg.disabled_servers[repo] = true
+            end
+            if ok then
+               break
+            else
+               util.warning("Failed searching manifest: "..err)
+            end
          end
       end
    end
    -- search through rocks in cfg.rocks_provided
    local provided_repo = "provided by VM or rocks_provided"
-   local name, versions
    for name, versions in pairs(cfg.rocks_provided) do
       store_if_match(results, provided_repo, name, versions, "installed", query)
    end
@@ -223,7 +236,7 @@ end
 -- @param name string: The query name.
 -- @param version string or nil: 
 -- @return table: A query in table format
-function make_query(name, version)
+function search.make_query(name, version)
    assert(type(name) == "string")
    assert(type(version) == "string" or not version)
    
@@ -267,28 +280,66 @@ local function pick_latest_version(name, versions)
    return nil
 end
 
---- Attempt to get a single URL for a given search.
--- @param query table: A dependency query.
--- @return string or table or (nil, string): URL for matching rock if
--- a single one was found, a table of candidates if it could not narrow to
--- a single result, or nil followed by an error message.
-function find_suitable_rock(query)
+-- Find out which other Lua versions provide rock versions matching a query,
+-- @param query table: A dependency query matching a single rock.
+-- @return table: array of Lua versions supported, in "5.x" format.
+local function supported_lua_versions(query)
+   local results = {}
+
+   for lua_version in util.lua_versions() do
+      if lua_version ~= cfg.lua_version then
+         if search.search_repos(query, lua_version)[query.name] then
+            table.insert(results, lua_version)
+         end
+      end
+   end
+
+   return results
+end
+
+--- Attempt to get a single URL for a given search for a rock.
+-- @param query table: A dependency query matching a single rock.
+-- @return string or (nil, string): URL for latest matching version
+-- of the rock if it was found, or nil followed by an error message.
+function search.find_suitable_rock(query)
    assert(type(query) == "table")
    
-   local results = search_repos(query)
-   local first = next(results)
-   if not first then
-      return nil, "No results matching query were found."
-   elseif not next(results, first) then
-      if cfg.rocks_provided[query.name] ~= nil then
-         -- do not install versions that listed in cfg.rocks_provided
-         return nil, "Rock "..query.name..
-                     " "..cfg.rocks_provided[query.name]..
-                     " was found but it is provided by VM or 'rocks_provided' in the config file."
+   local results = search.search_repos(query)
+   local first_rock = next(results)
+   if not first_rock then
+      if cfg.rocks_provided[query.name] == nil then
+         -- Check if constraints are satisfiable with other Lua versions.
+         local lua_versions = supported_lua_versions(query)
+
+         if #lua_versions ~= 0 then
+            -- Build a nice message in "only Lua 5.x and 5.y but not 5.z." format
+            for i, lua_version in ipairs(lua_versions) do
+               lua_versions[i] = "Lua "..lua_version
+            end
+
+            local versions_message = "only "..table.concat(lua_versions, " and ")..
+               " but not Lua "..cfg.lua_version.."."
+
+            if #query.constraints == 0 then
+               return nil, query.name.." supports "..versions_message
+            elseif #query.constraints == 1 and query.constraints[1].op == "==" then
+               return nil, query.name.." "..query.constraints[1].version.string.." supports "..versions_message
+            else
+               return nil, "Matching "..query.name.." versions support "..versions_message
+            end
+         end
       end
-      return pick_latest_version(query.name, results[first])
+
+      return nil, "No results matching query were found."
+   elseif next(results, first_rock) then
+      -- Shouldn't happen as query must match only one package.
+      return nil, "Several rocks matched query."
+   elseif cfg.rocks_provided[query.name] ~= nil then
+      -- Do not install versions listed in cfg.rocks_provided.
+      return nil, "Rock "..query.name.." "..cfg.rocks_provided[query.name]..
+         " was found but it is provided by VM or 'rocks_provided' in the config file."
    else
-      return results
+      return pick_latest_version(query.name, results[first_rock])
    end
 end
 
@@ -296,7 +347,7 @@ end
 -- @param results table: A table where keys are package names and versions
 -- are tables matching version strings to an array of rocks servers.
 -- @param porcelain boolean or nil: A flag to force machine-friendly output.
-function print_results(results, porcelain)
+function search.print_results(results, porcelain)
    assert(type(results) == "table")
    assert(type(porcelain) == "boolean" or not porcelain)
    
@@ -351,19 +402,51 @@ end
 -- @param name string: A rock name
 -- @param version string or nil: A version number may also be given.
 -- @return The result of the action function, or nil and an error message. 
-function act_on_src_or_rockspec(action, name, version, ...)
+function search.act_on_src_or_rockspec(action, name, version, ...)
    assert(type(action) == "function")
    assert(type(name) == "string")
    assert(type(version) == "string" or not version)
 
-   local query = make_query(name, version)
+   local query = search.make_query(name, version)
    query.arch = "src|rockspec"
-   local results, err = find_suitable_rock(query)
-   if type(results) == "string" then
-      return action(results, ...)
-   else
-      return nil, "Could not find a result named "..name..(version and " "..version or "").."."
+   local url, err = search.find_suitable_rock(query)
+   if not url then
+      return nil, "Could not find a result named "..name..(version and " "..version or "")..": "..err
    end
+   return action(url, ...)
+end
+
+function search.pick_installed_rock(name, version, given_tree)
+   local results = {}
+   local query = search.make_query(name, version)
+   query.exact_name = true
+   local tree_map = {}
+   local trees = cfg.rocks_trees
+   if given_tree then
+      trees = { given_tree }
+   end
+   for _, tree in ipairs(trees) do
+      local rocks_dir = path.rocks_dir(tree)
+      tree_map[rocks_dir] = tree
+      search.manifest_search(results, rocks_dir, query)
+   end
+
+   if not next(results) then --
+      return nil,"cannot find package "..name.." "..(version or "").."\nUse 'list' to find installed rocks."
+   end
+
+   version = nil
+   local repo_url
+   local package, versions = util.sortedpairs(results)()
+   --question: what do we do about multiple versions? This should
+   --give us the latest version on the last repo (which is usually the global one)
+   for vs, repositories in util.sortedpairs(versions, deps.compare_versions) do
+      if not version then version = vs end
+      for _, rp in ipairs(repositories) do repo_url = rp.repo end
+   end
+
+   local repo = tree_map[repo_url]
+   return name, version, repo, repo_url
 end
 
 --- Driver function for "search" command.
@@ -371,9 +454,7 @@ end
 -- @param version string or nil: a version may also be passed.
 -- @return boolean or (nil, string): True if build was successful; nil and an
 -- error message otherwise.
-function run(...)
-   local flags, name, version = util.parse_flags(...)
-   
+function search.command(flags, name, version)
    if flags["all"] then
       name, version = "", nil
    end
@@ -382,19 +463,21 @@ function run(...)
       return nil, "Enter name and version or use --all. "..util.see_help("search")
    end
    
-   local query = make_query(name:lower(), version)
+   local query = search.make_query(name:lower(), version)
    query.exact_name = false
-   local results, err = search_repos(query)
+   local results, err = search.search_repos(query)
    local porcelain = flags["porcelain"]
    util.title("Search results:", porcelain, "=")
    local sources, binaries = split_source_and_binary_results(results)
    if next(sources) and not flags["binary"] then
       util.title("Rockspecs and source rocks:", porcelain)
-      print_results(sources, porcelain)
+      search.print_results(sources, porcelain)
    end
    if next(binaries) and not flags["source"] then    
       util.title("Binary and pure-Lua rocks:", porcelain)
-      print_results(binaries, porcelain)
+      search.print_results(binaries, porcelain)
    end
    return true
 end
+
+return search
