@@ -4,11 +4,9 @@
 -- inside specific functions) to avoid interdependencies,
 -- as this is used in the bootstrapping stage of luarocks.cfg.
 
-local global_env = _G
+local util = {}
 
-module("luarocks.util", package.seeall)
-
-local cfg = require("luarocks.cfg")
+local unpack = unpack or table.unpack
 
 local scheduled_functions = {}
 local debug = require("debug")
@@ -20,7 +18,7 @@ local debug = require("debug")
 -- @param ... arguments to be passed to function.
 -- @return table: A token representing the scheduled execution,
 -- which can be used to remove the item later from the list.
-function schedule_function(f, ...)
+function util.schedule_function(f, ...)
    assert(type(f) == "function")
    
    local item = { fn = f, args = {...} }
@@ -32,7 +30,7 @@ end
 -- This is useful for cancelling a rollback of a completed operation.
 -- @param item table: The token representing the scheduled function that was
 -- returned from the schedule_function call.
-function remove_scheduled_function(item)
+function util.remove_scheduled_function(item)
    for k, v in pairs(scheduled_functions) do
       if v == item then
          table.remove(scheduled_functions, k)
@@ -46,7 +44,7 @@ end
 -- corresponding cleanup functions. Calling this function will run
 -- these function, erasing temporaries.
 -- Functions are executed in the inverse order they were scheduled.
-function run_scheduled_functions()
+function util.run_scheduled_functions()
    local fs = require("luarocks.fs")
    fs.change_dir_to_root()
    for i = #scheduled_functions, 1, -1 do
@@ -60,77 +58,163 @@ end
 -- so it does not include beginning- and end-of-string markers (^$)
 -- @param s string: The input string
 -- @return string: The equivalent pattern
-function matchquote(s)
+function util.matchquote(s)
    return (s:gsub("[?%-+*%[%].%%()$^]","%%%1"))
 end
 
+--- List of supported arguments.
+-- Arguments that take no parameters are marked with the boolean true.
+-- Arguments that take a parameter are marked with a descriptive string.
+-- Arguments that may take an empty string are described in quotes,
+-- (as in the value for --detailed="<text>").
+-- For all other string values, it means the parameter is mandatory.
+local supported_flags = {
+   ["all"] = true,
+   ["api-key"] = "<key>",
+   ["append"] = true,
+   ["arch"] = "<arch>",
+   ["bin"] = true,
+   ["binary"] = true,
+   ["branch"] = "<branch-name>",
+   ["debug"] = true,
+   ["deps"] = true,
+   ["deps-mode"] = "<mode>",
+   ["detailed"] = "\"<text>\"",
+   ["force"] = true,
+   ["force-fast"] = true,
+   ["from"] = "<server>",
+   ["help"] = true,
+   ["home"] = true,
+   ["homepage"] = "\"<url>\"",
+   ["keep"] = true,
+   ["lib"] = "<library>",
+   ["license"] = "\"<text>\"",
+   ["list"] = true,
+   ["local"] = true,
+   ["local-tree"] = true,
+   ["lr-bin"] = true,
+   ["lr-cpath"] = true,
+   ["lr-path"] = true,
+   ["lua-version"] = "<vers>",
+   ["lua-ver"] = true,
+   ["lua-incdir"] = true,
+   ["lua-libdir"] = true,
+   ["modules"] = true,
+   ["mversion"] = true,
+   ["no-refresh"] = true,
+   ["nodeps"] = true,
+   ["old-versions"] = true,
+   ["only-deps"] = true,
+   ["only-from"] = "<server>",
+   ["only-server"] = "<server>",
+   ["only-sources"] = "<url>",
+   ["only-sources-from"] = "<url>",
+   ["outdated"] = true,
+   ["output"] = "<file>",
+   ["pack-binary-rock"] = true,
+   ["porcelain"] = true,
+   ["quick"] = true,
+   ["rock-dir"] = true,
+   ["rock-tree"] = true,
+   ["rock-trees"] = true,
+   ["rockspec"] = true,
+   ["rockspec-format"] = "<ver>",
+   ["server"] = "<server>",
+   ["skip-pack"] = true,
+   ["source"] = true,
+   ["summary"] = "\"<text>\"",
+   ["system-config"] = true,
+   ["tag"] = "<tag>",
+   ["timeout"] = "<seconds>",
+   ["to"] = "<path>",
+   ["tree"] = "<path>",
+   ["user-config"] = true,
+   ["verbose"] = true,
+   ["version"] = true,
+}
+
 --- Extract flags from an arguments list.
 -- Given string arguments, extract flag arguments into a flags set.
 -- For example, given "foo", "--tux=beep", "--bla", "bar", "--baz",
 -- it would return the following:
 -- {["bla"] = true, ["tux"] = "beep", ["baz"] = true}, "foo", "bar".
-function parse_flags(...)
+function util.parse_flags(...)
    local args = {...}
    local flags = {}
-   for i = #args, 1, -1 do
+   local i = 1
+   local out = {}
+   local ignore_flags = false
+   while i <= #args do
       local flag = args[i]:match("^%-%-(.*)")
-      if flag then
+      if flag == "--" then
+         ignore_flags = true
+      end
+      if flag and not ignore_flags then
          local var,val = flag:match("([a-z_%-]*)=(.*)")
          if val then
-            flags[var] = val
+            local vartype = supported_flags[var]
+            if type(vartype) == "string" then
+               if val == "" and vartype:sub(1,1) ~= '"' then
+                  return { ERROR = "Invalid argument: parameter to flag --"..var.."="..vartype.." cannot be empty." }
+               end
+               flags[var] = val
+            else
+               if vartype then
+                  return { ERROR = "Invalid argument: flag --"..var.." does not take an parameter." }
+               else
+                  return { ERROR = "Invalid argument: unknown flag --"..var.."." }
+               end
+            end
          else
-            flags[flag] = true
+            local var = flag
+            local vartype = supported_flags[var]
+            if type(vartype) == "string" then
+               i = i + 1
+               local val = args[i]
+               if not val then
+                  return { ERROR = "Invalid argument: flag --"..var.."="..vartype.." expects a parameter." }
+               end
+               if val:match("^%-%-.*") then
+                  return { ERROR = "Invalid argument: flag --"..var.."="..vartype.." expects a parameter (if you really want to pass "..val.." as an argument to --"..var..", use --"..var.."="..val..")." }
+               else
+                  if val == "" and vartype:sub(1,1) ~= '"' then
+                     return { ERROR = "Invalid argument: parameter to flag --"..var.."="..vartype.." cannot be empty." }
+                  end
+                  flags[var] = val
+               end
+            elseif vartype == true then
+               flags[var] = true
+            else
+               return { ERROR = "Invalid argument: unknown flag --"..var.."." }
+            end
          end
-         table.remove(args, i)
+      else
+         table.insert(out, args[i])
       end
+      i = i + 1
    end
-   return flags, unpack(args)
+   return flags, unpack(out)
 end
 
---- Build a sequence of flags for forwarding from one command to
--- another (for example, from "install" to "build").
--- @param flags table: A table of parsed flags
--- @param ... string...: A variable number of flags to be checked
--- in the flags table. If no flags are passed as varargs, the
--- entire flags table is forwarded.
--- @return string... A variable number of strings
-function forward_flags(flags, ...)
-   assert(type(flags) == "table")
-   local out = {}
-   local filter = select('#', ...)
-   local function add_flag(flagname)
-      if flags[flagname] then
-         if flags[flagname] == true then
-            table.insert(out, "--"..flagname)
-         else
-            table.insert(out, "--"..flagname.."="..flags[flagname])
-         end
-      end
-   end
-   if filter > 0 then
-      for i = 1, filter do
-         add_flag(select(i, ...))
-      end
-   else
-      for flagname, _ in pairs(flags) do
-         add_flag(flagname)
-      end
-   end
-   return unpack(out)
+-- Adds legacy 'run' function to a command module.
+-- @param command table: command module with 'command' function,
+-- the added 'run' function calls it after parseing command-line arguments.
+function util.add_run_function(command)
+   command.run = function(...) return command.command(util.parse_flags(...)) end
 end
 
 --- Merges contents of src on top of dst's contents.
 -- @param dst Destination table, which will receive src's contents.
 -- @param src Table which provides new contents to dst.
 -- @see platform_overrides
-function deep_merge(dst, src)
+function util.deep_merge(dst, src)
    for k, v in pairs(src) do
       if type(v) == "table" then
          if not dst[k] then
             dst[k] = {}
          end
          if type(dst[k]) == "table" then
-            deep_merge(dst[k], v)
+            util.deep_merge(dst[k], v)
          else
             dst[k] = v
          end
@@ -153,7 +237,7 @@ end
 -- tbl.x are preserved).
 -- @param tbl table or nil: Table which may contain a "platforms" field;
 -- if it doesn't (or if nil is passed), this function does nothing.
-function platform_overrides(tbl)
+function util.platform_overrides(tbl)
    assert(type(tbl) == "table" or not tbl)
    
    local cfg = require("luarocks.cfg")
@@ -164,7 +248,7 @@ function platform_overrides(tbl)
       for _, platform in ipairs(cfg.platforms) do
          local platform_tbl = tbl.platforms[platform]
          if platform_tbl then
-            deep_merge(tbl, platform_tbl)
+            util.deep_merge(tbl, platform_tbl)
          end
       end
    end
@@ -178,7 +262,7 @@ local var_format_pattern = "%$%((%a[%a%d_]+)%)"
 -- the original table (ie, does not copy recursively).
 -- @param tbl table: the input table
 -- @return table: a new table with the same contents.
-local function make_shallow_copy(tbl)
+function util.make_shallow_copy(tbl)
    local copy = {}
    for k,v in pairs(tbl) do
       copy[k] = v
@@ -196,15 +280,15 @@ end
 -- @param needed_set: a set where keys are the names of
 -- needed variables.
 -- @param msg string: the warning message to display.
-function warn_if_not_used(var_defs, needed_set, msg)
-   needed_set = make_shallow_copy(needed_set)
-   for var,val in pairs(var_defs) do
+function util.warn_if_not_used(var_defs, needed_set, msg)
+   needed_set = util.make_shallow_copy(needed_set)
+   for _, val in pairs(var_defs) do
       for used in val:gmatch(var_format_pattern) do
          needed_set[used] = nil
       end
    end
-   for var,_ in pairs(needed_set) do
-      warning(msg:format(var))
+   for var, _ in pairs(needed_set) do
+      util.warning(msg:format(var))
    end
 end
 
@@ -215,7 +299,7 @@ local function warn_failed_matches(line)
    local any_failed = false
    if line:match(var_format_pattern) then
       for unmatched in line:gmatch(var_format_pattern) do
-         warning("unmatched variable " .. unmatched)
+         util.warning("unmatched variable " .. unmatched)
          any_failed = true
       end
    end
@@ -230,7 +314,7 @@ end
 -- @param tbl table: Table to have its string values modified.
 -- @param vars table: Table containing string-string key-value pairs 
 -- representing variables to replace in the strings values of tbl.
-function variable_substitutions(tbl, vars)
+function util.variable_substitutions(tbl, vars)
    assert(type(tbl) == "table")
    assert(type(vars) == "table")
    
@@ -251,7 +335,7 @@ end
 --- Return an array of keys of a table.
 -- @param tbl table: The input table.
 -- @return table: The array of keys.
-function keys(tbl)
+function util.keys(tbl)
    local ks = {}
    for k,_ in pairs(tbl) do
       table.insert(ks, k)
@@ -273,41 +357,6 @@ local function default_sort(a, b)
    end
 end
 
--- The iterator function used internally by util.sortedpairs.
--- @param tbl table: The table to be iterated.
--- @param sort_function function or nil: An optional comparison function
--- to be used by table.sort when sorting keys.
--- @see sortedpairs
-local function sortedpairs_iterator(tbl, sort_function)
-   local ks = keys(tbl)
-   if not sort_function or type(sort_function) == "function" then
-      table.sort(ks, sort_function or default_sort)
-      for _, k in ipairs(ks) do
-         coroutine.yield(k, tbl[k])
-      end
-   else
-      local order = sort_function
-      local done = {}
-      for _, k in ipairs(order) do
-         local sub_order
-         if type(k) == "table" then
-            sub_order = k[2]
-            k = k[1]
-         end
-         if tbl[k] then
-            done[k] = true
-            coroutine.yield(k, tbl[k], sub_order)
-         end
-      end
-      table.sort(ks, default_sort)
-      for _, k in ipairs(ks) do
-         if not done[k] then
-            coroutine.yield(k, tbl[k])
-         end
-      end
-   end
-end
-
 --- A table iterator generator that returns elements sorted by key,
 -- to be used in "for" loops.
 -- @param tbl table: The table to be iterated.
@@ -315,14 +364,56 @@ end
 -- to be used by table.sort when sorting keys, or an array listing an explicit order
 -- for keys. If a value itself is an array, it is taken so that the first element
 -- is a string representing the field name, and the second element is a priority table
--- for that key.
+-- for that key, which is returned by the iterator as the third value after the key
+-- and the value.
 -- @return function: the iterator function.
-function sortedpairs(tbl, sort_function)
-   return coroutine.wrap(function() sortedpairs_iterator(tbl, sort_function) end)
+function util.sortedpairs(tbl, sort_function)
+   sort_function = sort_function or default_sort
+   local keys = util.keys(tbl)
+   local sub_orders = {}
+
+   if type(sort_function) == "function" then
+      table.sort(keys, sort_function)
+   else
+      local order = sort_function
+      local ordered_keys = {}
+      local all_keys = keys
+      keys = {}
+
+      for _, order_entry in ipairs(order) do
+         local key, sub_order
+         if type(order_entry) == "table" then
+            key = order_entry[1]
+            sub_order = order_entry[2]
+         else
+            key = order_entry
+         end
+
+         if tbl[key] then
+            ordered_keys[key] = true
+            sub_orders[key] = sub_order
+            table.insert(keys, key)
+         end
+      end
+
+      table.sort(all_keys, default_sort)
+      for _, key in ipairs(all_keys) do
+         if not ordered_keys[key] then
+            table.insert(keys, key)
+         end
+      end
+   end
+
+   local i = 1
+   return function()
+      local key = keys[i]
+      i = i + 1
+      return key, tbl[key], sub_orders[key]
+   end
 end
 
-function lua_versions()
-   local versions = { "5.1", "5.2" }
+function util.lua_versions()
+   local versions = { "5.1", "5.2", "5.3" }
    local i = 0
    return function()
       i = i + 1
@@ -330,37 +421,37 @@ function lua_versions()
    end
 end
 
-function starts_with(s, prefix)
+function util.starts_with(s, prefix)
    return s:sub(1,#prefix) == prefix
 end
 
 --- Print a line to standard output
-function printout(...)
+function util.printout(...)
    io.stdout:write(table.concat({...},"\t"))
    io.stdout:write("\n")
 end
 
 --- Print a line to standard error
-function printerr(...)
+function util.printerr(...)
    io.stderr:write(table.concat({...},"\t"))
    io.stderr:write("\n")
 end
 
 --- Display a warning message.
 -- @param msg string: the warning message
-function warning(msg)
-   printerr("Warning: "..msg)
+function util.warning(msg)
+   util.printerr("Warning: "..msg)
 end
 
-function title(msg, porcelain, underline)
+function util.title(msg, porcelain, underline)
    if porcelain then return end
-   printout()
-   printout(msg)
-   printout((underline or "-"):rep(#msg))
-   printout()
+   util.printout()
+   util.printout(msg)
+   util.printout((underline or "-"):rep(#msg))
+   util.printout()
 end
 
-function this_program(default)
+function util.this_program(default)
    local i = 1
    local last, cur = default, default
    while i do
@@ -373,7 +464,8 @@ function this_program(default)
    return last:sub(2)
 end
 
-function deps_mode_help(program)
+function util.deps_mode_help(program)
+   local cfg = require("luarocks.cfg")
    return [[
 --deps-mode=<mode>  How to handle dependencies. Four modes are supported:
                     * all - use all trees from the rocks_trees list
@@ -386,18 +478,97 @@ function deps_mode_help(program)
                     The default mode may be set with the deps_mode entry
                     in the configuration file.
                     The current default is "]]..cfg.deps_mode..[[".
-                    Type ']]..this_program(program or "luarocks")..[[' with no arguments to see
+                    Type ']]..util.this_program(program or "luarocks")..[[' with no arguments to see
                     your list of rocks trees.
 ]]
 end
 
-function see_help(command, program)
-   return "See '"..this_program(program or "luarocks")..' help '..command.."'."
+function util.see_help(command, program)
+   return "See '"..util.this_program(program or "luarocks")..' help'..(command and " "..command or "").."'."
+end
+
+function util.announce_install(rockspec)
+   local cfg = require("luarocks.cfg")
+   local path = require("luarocks.path")
+
+   local suffix = ""
+   if rockspec.description and rockspec.description.license then
+      suffix = " (license: "..rockspec.description.license..")"
+   end
+
+   local root_dir = path.root_dir(cfg.rocks_dir)
+   util.printout(rockspec.name.." "..rockspec.version.." is now installed in "..root_dir..suffix)
+   util.printout()
+end
+
+--- Collect rockspecs located in a subdirectory.
+-- @param versions table: A table mapping rock names to newest rockspec versions.
+-- @param paths table: A table mapping rock names to newest rockspec paths.
+-- @param unnamed_paths table: An array of rockspec paths that don't contain rock
+-- name and version in regular format.
+-- @param subdir string: path to subdirectory.
+local function collect_rockspecs(versions, paths, unnamed_paths, subdir)
+   local fs = require("luarocks.fs")
+   local dir = require("luarocks.dir")
+   local path = require("luarocks.path")
+   local deps = require("luarocks.deps")
+
+   if fs.is_dir(subdir) then
+      for file in fs.dir(subdir) do
+         file = dir.path(subdir, file)
+
+         if file:match("rockspec$") and fs.is_file(file) then
+            local rock, version = path.parse_name(file)
+
+            if rock then
+               if not versions[rock] or deps.compare_versions(version, versions[rock]) then
+                  versions[rock] = version
+                  paths[rock] = file
+               end
+            else
+               table.insert(unnamed_paths, file)
+            end
+         end
+      end
+   end
+end
+
+--- Get default rockspec name for commands that take optional rockspec name.
+-- @return string or (nil, string): path to the rockspec or nil and error message.
+function util.get_default_rockspec()
+   local versions, paths, unnamed_paths = {}, {}, {}
+   -- Look for rockspecs in some common locations.
+   collect_rockspecs(versions, paths, unnamed_paths, ".")
+   collect_rockspecs(versions, paths, unnamed_paths, "rockspec")
+   collect_rockspecs(versions, paths, unnamed_paths, "rockspecs")
+
+   if #unnamed_paths > 0 then
+      -- There are rockspecs not following "name-version.rockspec" format.
+      -- More than one are ambiguous.
+      if #unnamed_paths > 1 then
+         return nil, "Please specify which rockspec file to use."
+      else
+         return unnamed_paths[1]
+      end
+   else
+      local rock = next(versions)
+
+      if rock then
+         -- If there are rockspecs for multiple rocks it's ambiguous.
+         if next(versions, rock) then
+            return nil, "Please specify which rockspec file to use."
+         else
+            return paths[rock]
+         end
+      else
+         return nil, "Argument missing: please specify a rockspec to use on current directory."
+      end
+   end
 end
 
 -- from http://lua-users.org/wiki/SplitJoin
 -- by PhilippeLhoste
-function split_string(str, delim, maxNb)
+function util.split_string(str, delim, maxNb)
    -- Eliminate bad cases...
    if string.find(str, delim) == nil then
       return { str }
@@ -426,12 +597,13 @@ end
 -- Example: given ("a;b;c;a;b;d", ";"), returns "a;b;c;d".
 -- @param list string: A path string (from $PATH or package.path)
 -- @param sep string: The separator
-function remove_path_dupes(list, sep)
+function util.remove_path_dupes(list, sep)
    assert(type(list) == "string")
    assert(type(sep) == "string")
-   local parts = split_string(list, sep)
+   local parts = util.split_string(list, sep)
    local final, entries = {}, {}
    for _, part in ipairs(parts) do
+      part = part:gsub("//", "/")
       if not entries[part] then
          table.insert(final, part)
          entries[part] = true
@@ -457,7 +629,7 @@ end
 -- @param name string: is the name of the table (optional)
 -- @param indent string: is a first indentation (optional).
 -- @return string: the pretty-printed table
-function show_table(t, name, indent)
+function util.show_table(t, name, indent)
    local cart     -- a container
    local autoref  -- for self references
 
@@ -523,7 +695,7 @@ function show_table(t, name, indent)
    return cart .. autoref
 end
 
-function array_contains(tbl, value)
+function util.array_contains(tbl, value)
    for _, v in ipairs(tbl) do
       if v == value then
          return true
@@ -535,6 +707,8 @@ end
 -- Quote Lua string, analogous to fs.Q.
 -- @param s A string, such as "hello"
 -- @return string: A quoted string, such as '"hello"'
-function LQ(s)
+function util.LQ(s)
    return ("%q"):format(s)
 end
+
+return util
